library(plyr)
library(dplyr)
data <- read.csv('data.csv') %>% within({
	contrasts(gender) <- contr.sum
	contrasts(following) <- contr.SAS
	region <- factor(region,levels=c('NM','NN','NS','FB','FL','FE','FW','NR'))
	contrasts(region) <- contr.sum
	region_following <- interaction(region,following)
})

################################################################################
# Full models
################################################################################
# Au does not converge
data.au <- data[data$vowel == 'Au' & data$formant == 'f1' & data$value > 100 & data$value < 1000,]
m <- mgcv::bam(value ~ gender + region +
	       s(time,by=region,k=30,bs='tp') +
	       s(participant,bs='re') + s(time,participant,k=30,bs='fs',xt='tp',m=1),
               data=data.au,nthreads=40,discrete=T,control=list(trace=T,maxit=9999),family=mgcv::scat,rho=.5,AR.start=data.au$time==0)
save('m',file='out/f1-Au.RData',compress='xz')
data <- data[data$vowel != 'Au',]
fit <- function (data,dep) {
	cat('================================================================================\n')
	cat(v <- as.character(data$vowel[1]))
	cat('\n')
	cat('================================================================================\n')
	data <- data[data$formant == dep,]
	if (dep == 'f1') data <- data[data$value > 100 & data$value < 1000,]
	if (dep == 'f2') data <- data[data$value > 500 & data$value < 3000,]
	f <- value ~ gender + region*following +
		s(time,by=region_following,k=k,bs='tp') +
		s(participant,bs='re',by=following) + s(time,participant,k=k,bs='fs',xt='tp',m=1,by=following)
	k <- 30
	m <- mgcv::bam(f,data=data,nthreads=40,discrete=T,control=list(trace=T,maxit=9999),family=mgcv::scat,rho=.5,AR.start=data$time==0)
	save('m',file=paste0('out/',dep,'-',v,'.RData'),compress='xz')
}
d_ply(data,~vowel,fit,'f1')
d_ply(data,~vowel,fit,'f2')

################################################################################
# By-word models
################################################################################

fit <- function (data,dep) {
	cat('================================================================================\n')
	cat(v <- as.character(data$vowel[1]))
	cat(wd <- as.character(data$word[1]))
	cat('\n')
	cat('================================================================================\n')
	data <- data[data$formant == dep,]
	if (dep == 'f1') data <- data[data$value > 100 & data$value < 1000,]
	if (dep == 'f2') data <- data[data$value > 500 & data$value < 3000,]
	f <- value ~ gender + region +
		s(time,by=region,k=k,bs='tp') +
		s(participant,bs='re') + s(time,participant,k=k,bs='fs',xt='tp',m=1)
	k <- 10
	m <- mgcv::bam(f,data=data,nthreads=40,discrete=T,control=list(trace=T,maxit=9999),family=mgcv::scat,rho=.5,AR.start=data$time==0)
	save('m',file=paste0('out/',dep,'-',v,'-',wd,'.RData'),compress='xz')
}
d_ply(data,~word,fit,'f1')
d_ply(data,~word,fit,'f2')

################################################################################
# Obtain predictions
################################################################################

library(parallel)
cl <- makeCluster(10,outfile='')
ff <- paste0('out/',dir('out/'))
xt <- parLapply(cl,ff,function (f) {
	library(mgcv)
	library(plyr)
	library(dplyr)
	load(f)
	formant <- sub('.*f','',f)
	formant <- substr(formant,1,1)
	is.word <- nchar(gsub('[^-]','',f)) > 1

	lpfun <- function (cond) {
		combs <- list(list(gender='m'),list(gender='f'))
		res <- lapply(combs,function (comb) {
			cond <- c(cond,comb,list(participant='NRvo1'))
			newdata <- expand.grid(cond)
			lp <- mgcv:::predict.gam(m,newdata,type='lpmatrix')
			lp[,grepl('participant',colnames(lp))] <- 0
			lp
		})
		Reduce('+',res) / length(combs)
	}
	lp2df <- function (lp,...) {
		y <- lp %*% coef(m)
		ci <- qnorm(.025,lower.tail=F) * sqrt(rowSums((lp %*% vcov(m)) * lp))
		data.frame(...,y=y,CI=ci)
	}
	WRAP <- function (fun) adply(as.character(.(NR,NM,NN,NS,FB,FL,FE,FW)),1,function (x) {
		cat('Region =',x,'\n')
		df <- fun(x)
		df$region <- x
		df
	},.id=NULL)
	L <- c(P='P',l='l')
	P <- function (x,y) paste0(x,'.',y)

	cat('1. Fits -- for both /l/ and non-/l/\n')
	preds <- WRAP(function (x) adply(L,1,function (l) lp2df(lpfun(list(time=0:100,region=x,following=l,region_following=P(x,l))),time=0:100),.id='following'))

	cat('2. Regional differences: l\n')
	Randstad.l <- lpfun(list(time=0:100,region='NR',following='l',region_following='NR.l'))
	diff.l <- WRAP(function (x) {
		me <- lpfun(list(time=0:100,region=x,following='l',region_following=P(x,'l')))
		lp2df(me - Randstad.l,time=0:100)
	})

	# For the by-word models, P and l are not used (argument 'following' will be ignored silently, resulting in duplicated prediction data) and we now have all we need
	# For the F2 models, we only need following='l'
	if (is.word || formant == 'f2') {
		cat('Early exit for',f,'\n')
		return(list(preds=preds[preds$following == 'l',],diff=diff.l))
	}

	cat('3. Regional differences: P\n')
	Randstad.P <- lpfun(list(time=0:100,region='NR',following='P',region_following='NR.P'))
	diff.P <- WRAP(function (x) {
		me <- lpfun(list(time=0:100,region=x,following='P',region_following=P(x,'P')))
		lp2df(me - Randstad.P,time=0:100)
	})

	cat('4. Difference between /l/ and non-/l/\n')
	# define baseline and contrasting following-segment categories
	base <- if (formant == 1) 'l' else 'P'
	cont <- if (formant == 1) 'P' else 'l'
	cat('4.1. Identify peaks\n')
	peakdata <- group_by(preds,region,following)
	peaks    <- filter(peakdata,time > 50) %>% filter(y == min(y))
	troughs  <- filter(peakdata,time < 50) %>% filter(y == max(y))
	cat('4.2. Generate lpmatrices and fits for the trough-to-peak ranges for /l/ and non-/l/\n')
	ttp <- group_by(peaks,region,following) %>% group_map(function (.x,.y) {
		time.peak   <- left_join(.y,peaks)$time
		time.trough <- left_join(.y,troughs)$time
		lp.peak   <- lpfun(list(time=time.peak,region=.y$region,following=.y$following,region_following=P(.y$region,.y$following)))
		lp.trough <- lpfun(list(time=time.trough,region=.y$region,following=.y$following,region_following=P(.y$region,.y$following)))
		lp <- lp.peak - lp.trough
		list(key=.y,value=lp)
	})
	cat('4.3. Difference /l/ and non-/l/\n')
	ttpdiff <- lapply(as.character(.(NR,NM,NN,NS,FB,FL,FE,FW)),function (x) {
		needles <- lapply(ttp,function (y) if (y$key$region == x) y else NULL)
		needles <- needles[!sapply(needles,is.null)]
		names(needles) <- sapply(needles,function (x) x$key$following)
		base <- needles[[base]]$value
		cont <- needles[[cont]]$value
		cont - base
	})
	names(ttpdiff) <- as.character(.(NR,NM,NN,NS,FB,FL,FE,FW))
	cat('4.4. That difference compared to the Randstad\n')
	ttpNR <- WRAP(function (x) lp2df(ttpdiff[[x]] - ttpdiff[['NR']]))
	cat('4.5. Also generate a df for the ttpdiff itself\n')
	ttpdiff <- ldply(ttpdiff,lp2df,.id='region')

	list(preds=preds,diff.P=diff.P,diff.l=diff.l,ttpdiff=ttpdiff,ttpNR=ttpNR)
})
stopCluster(cl)
names(xt) <- sub('.RData','',ff)
names(xt) <- sub('out/','',names(xt))
save('xt',file='plotdata-gam.RData',compress='xz')

library(magrittr)
library(dplyr)
library(tidyr)
cifun <- function (y,CI,fun) ifelse(abs(y) > CI,fun(y,CI),NA)
recoder <- function (x,...) recode(x,...) %>% factor(levels=unlist(list(...)))
d <- ldply(xt,function (x) ldply(x,identity,.id='cond'),.id='model') %>%
	separate(model,c('formant','vowel','word'),'-') %>%
	mutate(cimin  = cifun(y,CI,`-`), cimax = cifun(y,CI,`+`),
		Region = factor(region,levels=c('NR','NM','NN','NS','FB','FL','FE','FW')),
		vowel  = recoder(vowel,'e:'='\\textipa{e:}','2:'='\\textipa{\\o:}','o:'='\\textipa{o:}','Ei'='\\textipa{Ei}','9y'='\\textipa{\\oe y}','Au'='\\textipa{Au}'),
		word = as.character(word),
		cond = as.character(cond)
	) %>% filter(vowel != '\\textipa{Au}')
d[is.na(d$word) & d$cond == 'diff.P',] %<>% mutate(word = 'P', cond = 'diff')
d[is.na(d$word) & d$cond == 'diff.l',] %<>% mutate(word = 'l', cond = 'diff')

library(ggplot2)
filter(d,word %in% c('P','l')) %>% dlply(~formant+cond+word,function (part) {
	cond <- part$cond[1]
	f <- part$formant[1]
	p <- ggplot(data=part,aes(time,y,color=Region,group=Region,fill=Region)) + facet_wrap(~vowel,nrow=3)
	if (cond != 'preds') p <- p + geom_hline(yintercept=0,color='black')
	p <- p + geom_line()
	p <- p + geom_ribbon(aes(ymin=cimin,ymax=cimax,linetype=NA),alpha=.1)
	p <- p + theme_bw() + scale_x_continuous(expand=c(0,0))
	if (f == 'f1') p <- p + scale_y_reverse()
	p <- p + xlab('Time (\\%)') + ylab(paste0('F',substr(f,2,2),' (Hz)'))
	p
}) -> p

################################################################################
# Generate data for tables
################################################################################
d %<>% group_by(formant,vowel,word,cond,Region) %>% group_modify(function (piece,key) {
	# Stretches of signal within the first/last 50% are identified semi-automatically. The 'semi' part is set up here. The algorithm used later *mostly* selects the right stretches of signal for testing/comparing, but not always; these are excluded on the basis of the 'runs' identifier set up here.
	runs <- rle(is.na(piece$cimin))$lengths
	names(runs) <- paste(key,collapse='.') %>% paste(1:length(runs))
	runs <- names(runs) %>% lapply(function (n) rep(n,runs[[n]])) %>% unlist
	transmute(piece,runs=runs,region=region,following=following,time=time,y=y,CI=CI,cimin=cimin,cimax=cimax)
})
fetch <- function (d,filter1,filter2,filter3)
	filter3(d) %>%
	group_by(vowel,region,word) %>%
	filter1 %>% filter2 %>%
	group_modify(function (piece,key) {
		piece <- piece[1,]
		fullpiece <- filter(d,runs == piece$runs)
		mutate(piece, min = min(fullpiece$time), max = max(fullpiece$time))
	})
compute <- function (filter1,filter2,filter3,filter4) fetch(d,filter1,filter2,filter3) %>%
	# lexical diffusion
	group_by(vowel,region) %>%
	group_modify(function (piece,key) {
		words <- fetch(d,filter1,filter2,filter4) %>% right_join(key)
		df <- nrow(words) - 1
		if (df < 1) return(piece)
		VarExp <- sum((piece$y - words$y)^2)
		VarRes <- (piece$CI / qnorm(.025,lower.tail=F))^2
		X2 <- VarExp / VarRes
		mutate(piece,Chisq=X2,p=pchisq(X2,df,lower.tail=F))
	}) %>%
	ungroup %>%
	arrange(vowel,Region) %>%
	as.data.frame

change1 <- compute(
	. %>% filter(time > 50, !runs %in% c('f1.2.P.diff.4 1','f1.4.P.diff.3 2','f1.5.P.diff.4 3','f1.2.P.diff.7 4','f1.2.P.diff.7 2')),
	. %>% filter(y == max(y)),
	. %>% filter(formant == 'f1', cond == 'diff', !is.na(cimin), word == 'P'),
	. %>% filter(formant == 'f1', cond == 'diff', !word %in% c('P','l'), !endsWith(word,'l'))
)
change2 <- compute(
	. %>% filter(time < 50, time > 10, !runs %in% c('f1.1.P.diff.5 3','f1.4.P.diff.5 2')),
	. %>% filter(y == min(y)),
	. %>% filter(formant == 'f1', cond == 'diff', !is.na(cimin), word == 'P'),
	. %>% filter(formant == 'f1', cond == 'diff', !word %in% c('P','l'), !endsWith(word,'l'))
)
change3 <- filter(d, formant == 'f1', cond == 'ttpNR', !is.na(cimin)) %>%
	group_by(vowel,region) %>%
	group_modify(function (piece,key) {
		# 1. All words have ttps, how does each word's individual ttp differ from the ttp from the Randstad?
		#    Results in two sets of explained variance and df: one for /l/ and one for non-/l/
		# 2. Sum them
		Randstad <- filter(d, formant == 'f1', cond == 'ttpdiff', is.na(word), region == 'NR', vowel == key$vowel)
		peakdata <- filter(d, formant == 'f1', cond == 'preds', !is.na(word)) %>%
			right_join(key) %>%
			group_by(word) %>%
			# Compute ttp for each word
			group_modify(function (piece,key) {
				peak     <- filter(piece,time > 50) %>% filter(y == min(y))
				trough   <- filter(piece,time < 50) %>% filter(y == max(y))
				data.frame(ttp=peak$y-trough$y)
			}) %>%
			mutate(following = ifelse(endsWith(word,'l'),'l','P')) %>%
			ungroup
		# Compute differences from reference ttps for /l/ and non-/l/ by forming combinations
		P <- filter(peakdata,following == 'P')
		l <- filter(peakdata,following == 'l')
		diffs <- unlist(lapply(P$ttp,function (x) x - l$ttp))
		if (!length(diffs)) return(piece)
		VarExp <- sum((diffs - Randstad$y)^2)
		VarRes <- (Randstad$CI / qnorm(.025,lower.tail=F))^2
		X2 <- VarExp / VarRes
		df <- length(diffs) - 1
		mutate(piece,Chisq=X2,p=pchisq(X2,df,lower.tail=F))
	}) %>%
	# Add the original ttps
	group_modify(function (piece,key) filter(d, formant == 'f1', cond == 'ttpdiff', vowel == key$vowel, region == key$region) %>%
		with(data.frame(ttp.y=y,ttp.cimin=y-CI,ttp.cimax=y+CI)) %>%
		cbind(piece)
	) %>%
	as.data.frame
change4 <- compute(
	. %>% filter(time > 50, time < 90),
	. %>% filter(y == max(y)),
	. %>% filter(formant == 'f2', cond == 'diff', !is.na(cimin), word == 'l'),
	. %>% filter(formant == 'f2', cond == 'diff', !word %in% c('P','l'), endsWith(word,'l'))
)

neground1 <- function (x,n) ifelse(x < 0,
	paste0('\\texttt{-}',-round(x,n)),
	round(x,n)
)
neground2 <- function (x,n) ifelse(x < 0,
	paste0('\\texttt{-}',-round(x,n)),
	paste0('\\hphantom{\\texttt{-}}',round(x,n))
)
tex <- . %>% transmute(vowel = vowel, region = region, time = paste(min,max,sep=' -- '), peak = neground2(y,2), ci = paste(neground2(cimin,2),neground1(cimax,2),sep=' -- '), Chisq = round(Chisq,2), p = round(p,2)) %>% apply(1,paste,collapse=' & ') %>% paste(collapse='\\\\\n') %>% cat
tex(change1)
tex(change2)
change3 %>% transmute(vowel = vowel, region = region, ttp.y=neground2(ttp.y,2), ttp.ci = paste(neground2(ttp.cimin,2),neground1(ttp.cimax,2),sep=' -- '), deltaNR = neground2(y,2), ci = paste(neground2(cimin,2),neground1(cimax,2),sep=' -- '), Chisq = round(Chisq,2), p = round(p,2)) %>% apply(1,paste,collapse=' & ') %>% paste(collapse='\\\\\n') %>% cat
tex(change4)
